@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: protocols.c,v 1.19 2002/11/27 14:29:55 masneyb Exp $";
+static const char cvsid[] = "$Id: protocols.c,v 1.21 2002/12/05 00:43:06 masneyb Exp $";
 
 gftp_request *
 gftp_request_new (void)
@@ -496,7 +496,7 @@ gftp_set_data_type (gftp_request * request, int data_type)
   g_return_val_if_fail (request != NULL, -2);
 
   if (request->set_data_type == NULL)
-    return (-2);
+    return (0);
   return (request->set_data_type (request, data_type));
 }
 
@@ -1588,9 +1588,6 @@ gftp_get_file_transfer_mode (char *filename, int def)
   GList * templist;
   int stlen, ret;
 
-  if (!use_default_dl_types)
-    return (def);
-
   ret = def;
   stlen = strlen (filename);
   for (templist = registered_exts; templist != NULL; templist = templist->next)
