@@ -20,7 +20,12 @@
 /* This will use Brian Wellington <bwelling@xbill.org>'s sftpserv program */
 /* on the remote. Some of this code is derived from the sftp client */
 
+/* 11/5/2002 - This protocol is now obsolete by the SSH2 protocol. I'm not 
+   going to be adding any new features to it, but I'll keep it inside the code 
+   for a little bit. I do plan on removing it completely though */
+
 #include "gftp.h"
+static const char cvsid[] = "$Id: ssh.c,v 1.10 2002/11/23 13:45:04 masneyb Exp $";
 
 #define CHDIR		10
 #define GETDIR		11
@@ -62,104 +67,210 @@ typedef struct ssh_message_tag
 } ssh_message;
 
 
-static void ssh_destroy 		( gftp_request * request );
-static int ssh_connect 			( gftp_request * request );
-static void ssh_disconnect 		( gftp_request * request );
-static long ssh_get_file 		( gftp_request * request, 
-					  const char *filename,
-					  FILE * fd,
-					  off_t startsize );
-static int ssh_put_file 		( gftp_request * request, 
-					  const char *filename,
-					  FILE * fd,
-					  off_t startsize,
-					  off_t totalsize );
-static size_t ssh_get_next_file_chunk 	( gftp_request * request, 
-					  char *buf, 
-					  size_t size );
-static size_t ssh_put_next_file_chunk 	( gftp_request * request, 
-					  char *buf, 
-					  size_t size );
-static int ssh_end_transfer 		( gftp_request * request );
-static int ssh_list_files 		( gftp_request * request );
-static int ssh_get_next_file 		( gftp_request * request, 
-					  gftp_file * fle, 
-					  FILE * fd );
-static int ssh_chdir 			( gftp_request * request, 
-					  const char *directory );
-static char *ssh_exec			( gftp_request * request,
-					  const char *command,
-					  size_t len);
-static int ssh_rmdir			( gftp_request * request, 
-					  const char *directory );
-static int ssh_rmfile			( gftp_request * request, 
-					  const char *file );
-static int ssh_mkdir			( gftp_request * request, 
-					  const char *newdir );
-static int ssh_rename			( gftp_request * request,
-					  const char *oldname,
-					  const char *newname );
-static int ssh_chmod			( gftp_request * request,
-					  const char *file,
-					  int mode );
-static int ssh_send_command 		( gftp_request * request, 
-					  int cmdnum, 
-					  const char *command, 
-					  size_t len );
-static int ssh_read_response 		( gftp_request * request, 
-					  ssh_message *message,
-					  FILE * fd );
-static char *ssh_read_message 		( gftp_request * request, 
-					  char *buf, 
-					  size_t len,
-					  FILE * fd );
-static char *ssh_read_line 		( gftp_request * request );
-static void ssh_log_command 		( gftp_request * request,
-					  int channel,
-					  int cmdnum, 
-					  const char *command, 
-					  size_t len,
-					  int direction );
-static size_t ssh_remove_spaces 	( char *string );
-
-void
-ssh_init (gftp_request * request)
+static void
+ssh_log_command (gftp_request * request, int channel, int cmdnum, 
+                 const char *command, size_t len, int direction)
 {
-  g_return_if_fail (request != NULL);
+  const char *pos;
+  char *tempstr;
+  int ok;
 
-  request->protonum = GFTP_SSH_NUM;
-  request->init = ssh_init;
-  request->destroy = ssh_destroy;
-  request->connect = ssh_connect;
-  request->disconnect = ssh_disconnect;
-  request->get_file = ssh_get_file;
-  request->put_file = ssh_put_file;
-  request->transfer_file = NULL;
-  request->get_next_file_chunk = ssh_get_next_file_chunk;
-  request->put_next_file_chunk = ssh_put_next_file_chunk;
-  request->end_transfer = ssh_end_transfer;
-  request->list_files = ssh_list_files;
-  request->get_next_file = ssh_get_next_file;
-  request->set_data_type = NULL;
-  request->get_file_size = NULL;
-  request->chdir = ssh_chdir;
-  request->rmdir = ssh_rmdir;
-  request->rmfile = ssh_rmfile;
-  request->mkdir = ssh_mkdir;
-  request->rename = ssh_rename;
-  request->chmod = ssh_chmod;
-  request->set_file_time = NULL;
-  request->site = NULL;
-  request->parse_url = NULL;
-  request->url_prefix = "ssh";
-  request->protocol_name = "SSH";
-  request->need_hostport = 1;
-  request->need_userpass = ssh_need_userpass;
-  request->use_cache = 1;
-  request->use_threads = 1;
-  request->always_connected = 0;
-  request->protocol_data = g_malloc0 (sizeof (ssh_parms));
-  gftp_set_config_options (request);
+  switch (cmdnum)
+    {
+      case CHDIR:
+        tempstr = "CHDIR ";
+        break;
+      case GETDIR:
+        tempstr = "GETDIR ";
+        break;
+      case TELLDIR:
+        tempstr = "TELLDIR ";
+        break;
+      case SENDFILE:
+        tempstr = "SENDFILE ";
+        break;
+      case FILESIZE:
+        tempstr = "FILESIZE ";
+        break;
+      case FILEMODE:
+        tempstr = "FILEMODE ";
+        break;
+      case ENDDATA:
+        tempstr = "ENDDATA ";
+        break;
+      case FILEOK:
+        tempstr = "FILEOK ";
+        break;
+      case STREAM:
+        tempstr = "STREAM ";
+        break;
+      case REQUEST:
+        tempstr = "REQUEST ";
+        break;
+      case FILENAME:
+        tempstr = "FILENAME ";
+        break;
+      case EXEC:
+        tempstr = "EXEC ";
+        break;
+      case SKIPBYTES:
+        tempstr = "SKIPBYTES ";
+        break;
+      case ERROR:
+        tempstr = "ERROR: ";
+        break;
+      case SUCCESS:
+        tempstr = "SUCCESS ";
+        break;
+      case CLOSE:
+        tempstr = "CLOSE ";
+        break;
+      case SSH_VERSION:
+        tempstr = "VERSION ";
+        break;
+      case CANCEL:
+        tempstr = "CANCEL ";
+        break;
+      case FILETIME:
+        tempstr = "FILETIME ";
+        break;
+      default:
+        return;
+    }
+
+  ok = 0;
+  if (command)
+    {
+      for (pos = command; pos < command + len; pos++)
+        {
+          if (*pos == '\0')
+            {
+              ok = 1;
+              break;
+            }
+        }
+    }
+
+  request->logging_function (direction == GFTP_DIRECTION_DOWNLOAD ? 
+                                 gftp_logging_send : gftp_logging_recv,
+                             request->user_data, "%d: %s %s\n", channel, 
+                             tempstr, ok ? command : "");
+}
+
+
+static int
+ssh_send_command (gftp_request * request, int cmdnum, const char *command, 
+                  size_t len)
+{
+  ssh_parms * params;
+  char *buf;
+  int clen;
+
+  params = request->protocol_data;
+  clen = htonl (len);
+  buf = g_malloc (len + 6);
+  buf[0] = params->channel;
+  buf[1] = cmdnum;
+  memcpy (&buf[2], &clen, 4);
+  if (command)
+    memcpy (&buf[6], command, len);
+  ssh_log_command (request, params->channel, cmdnum, command, len, 1);
+
+  if (gftp_write (request, buf, len + 6, request->sockfd) < 0)
+    return (-2);
+
+  return 0;
+
+}
+
+
+static char *
+ssh_read_message (gftp_request * request, char *buf, size_t len, int fd)
+{
+  if (fd <= 0)
+    fd = request->sockfd;
+
+  if (gftp_read (request, buf, len, fd) < 0)
+    return (NULL);
+
+  return (buf);
+}
+
+
+static int
+ssh_read_response (gftp_request * request, ssh_message *message, int fd)
+{
+  char buf[6];
+
+  if (ssh_read_message (request, buf, 6, fd) == NULL)
+    return (-1);
+
+  message->channel = buf[0];
+  message->command = buf[1];
+  memcpy (&message->len, buf + 2, 4);
+  message->len = ntohl (message->len);
+  if (message->len > 8192)
+    {
+      request->logging_function (gftp_logging_error, request->user_data,
+                             _("Error: Message size %d too big from server\n"),
+                             message->len);
+      memset (message, 0, sizeof (*message));
+      gftp_disconnect (request);
+      return (-1);
+    }
+  
+  message->data = g_malloc (message->len + 1); 
+
+  if (message->len > 0 && ssh_read_message (request, 
+                           (char *) message->data, message->len, fd) == NULL)
+    return (-1);
+
+  ((char *) message->data)[message->len] = '\0'; 
+  ssh_log_command (request, message->channel, message->command, message->data,
+                   message->len, 0);
+  return (message->command);
+}
+
+
+static char *
+ssh_read_line (gftp_request * request)
+{
+  char *retstr, *pos, tempchar;
+  ssh_parms *buffer;
+
+  pos = NULL;
+  buffer = request->protocol_data;
+  if (!buffer->enddata && (pos = strchr (buffer->pos, '\n')) == NULL)
+    return (NULL);
+
+  if (pos == NULL)
+    {
+      pos = buffer->pos + strlen (buffer->pos) - 1;
+      tempchar = '\0';
+    }
+  else
+    tempchar = pos + 1 == '\0' ? '\0' : '1';
+
+  if (*(pos-1) == '\r') 
+    *(pos-1) = '\0';
+  else 
+    *pos = '\0';
+
+  retstr = g_malloc (strlen (buffer->pos) + 1);
+  strcpy (retstr, buffer->pos);
+
+  if (tempchar != '\0' && *buffer->pos != '\0')
+    {
+      buffer->pos = pos + 1;
+      while (*buffer->pos == '\r' || *buffer->pos == '\n')
+        buffer->pos++;
+    }
+  else
+    {
+      g_free (buffer->buffer);
+      buffer->buffer = buffer->pos = NULL;
+    }
+  return (retstr);
 }
 
 
@@ -180,13 +291,61 @@ ssh_destroy (gftp_request * request)
 }
 
 
+static int
+ssh_chdir (gftp_request * request, const char *directory)
+{
+  ssh_message message;
+  int ret;
+
+  g_return_val_if_fail (request != NULL, -2);
+  g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
+
+  if (directory != NULL && *directory != '\0')
+    {  
+      if (ssh_send_command (request, CHDIR, directory, 
+                            strlen (directory) + 1) < 0) 
+        return (-1);
+
+      if ((ret = ssh_read_response (request, &message, -1)) != SUCCESS)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+			    _("Could not change remote directory to %s: %s\n"),
+	  	            directory, (char *) message.data);
+          g_free (message.data);
+          return (-1);
+        }
+      g_free (message.data);
+    }
+
+  if (directory != request->directory)
+    {
+      if (ssh_send_command (request, GETDIR, NULL, 0) < 0)
+        return (-1);
+
+      if (ssh_read_response (request, &message, -1) != TELLDIR)
+        {
+          request->logging_function (gftp_logging_error, request->user_data,
+                            _("Could not get current working directory: %s\n"),
+		  	    (char *) message.data);
+          g_free (message.data);
+          return (-1);
+        }
+
+      if (request->directory)
+        g_free (request->directory);
+      request->directory = message.data;
+    }
+
+  return (0);
+}
+
+
 static int
 ssh_connect (gftp_request * request)
 {
   char **args, *tempstr, pts_name[20], *p1, p2, *exepath, port[6];
   int version, fdm, fds, s[2];
   ssh_message message;
-  const gchar *errstr;
   ssh_parms *params;
   pid_t child;
 
@@ -195,7 +354,7 @@ ssh_connect (gftp_request * request)
   g_return_val_if_fail (request->hostname != NULL, -2);
   
   params = request->protocol_data;
-  if (request->sockfd != NULL)
+  if (request->sockfd > 0)
     return (0);
 
   request->logging_function (gftp_logging_misc, request->user_data,
@@ -268,11 +427,7 @@ ssh_connect (gftp_request * request)
       execvp (ssh_prog_name != NULL && *ssh_prog_name != '\0' ?
               ssh_prog_name : "ssh", args);  
 
-      tempstr = _("Error: Cannot execute ssh: ");
-      write (1, tempstr, strlen (tempstr));
-      errstr = g_strerror (errno);
-      write (1, errstr, strlen (errstr));
-      write (1, "\n", 1);
+      printf (_("Error: Cannot execute ssh: %s\n"), g_strerror (errno));
       return (-1);
     }
   else if (child > 0)
@@ -288,8 +443,6 @@ ssh_connect (gftp_request * request)
           !(strlen (tempstr) > 4 && strcmp (tempstr + strlen (tempstr) - 5, 
                                             "xsftp") == 0))
         {
-          request->logging_function (gftp_logging_error, request->user_data,
-		_("Error: Received wrong init string from server\n"));
           g_free (args);
           g_free (exepath);
           return (-2);
@@ -298,14 +451,13 @@ ssh_connect (gftp_request * request)
       g_free (exepath);
       g_free (tempstr);
 
-      request->sockfd = request->datafd = fdopen (fdm, "rb");
-      request->sockfd_write = fdopen (fdm, "wb");
+      request->sockfd = fdm;
 
       params->channel = 0;
       version = htonl (7 << 4);
       if (ssh_send_command (request, SSH_VERSION, (char *) &version, 4) < 0)
         return (-2);
-      if (ssh_read_response (request, &message, NULL) != SSH_VERSION)
+      if (ssh_read_response (request, &message, -1) != SSH_VERSION)
         return (-2);
       g_free (message.data);
 
@@ -327,7 +479,7 @@ ssh_connect (gftp_request * request)
   if (ssh_send_command (request, GETDIR, NULL, 0) < 0)
     return (-1);
 
-  if (ssh_read_response (request, &message, NULL) != TELLDIR)
+  if (ssh_read_response (request, &message, -1) != TELLDIR)
     {
       request->logging_function (gftp_logging_error, request->user_data,
                             _("Could not get current working directory: %s\n"),
@@ -340,9 +492,6 @@ ssh_connect (gftp_request * request)
     g_free (request->directory);
   request->directory = message.data;
 
-  if (request->sockfd == NULL)
-    return (-2);
-
   return (0);
 }
 
@@ -353,19 +502,24 @@ ssh_disconnect (gftp_request * request)
   g_return_if_fail (request != NULL);
   g_return_if_fail (request->protonum == GFTP_SSH_NUM);
 
-  if (request->sockfd != NULL)
+  if (request->sockfd > 0)
     {
       request->logging_function (gftp_logging_misc, request->user_data,
 			         _("Disconnecting from site %s\n"),
                                  request->hostname);
-      fclose (request->sockfd);
-      request->sockfd = request->datafd = request->sockfd_write = NULL;
+
+      if (close (request->sockfd) < 0)
+        request->logging_function (gftp_logging_error, request->user_data,
+                                   _("Error closing file descriptor: %s\n"),
+                                   g_strerror (errno));
+
+      request->sockfd = -1;
     }
 }
 
 
-static long
-ssh_get_file (gftp_request * request, const char *filename, FILE * fd,
+static off_t 
+ssh_get_file (gftp_request * request, const char *filename, int fd,
               off_t startsize)
 {
   ssh_message message;
@@ -386,7 +540,7 @@ ssh_get_file (gftp_request * request, const char *filename, FILE * fd,
   retsize = 0;
   while (1)
     {
-      ret = ssh_read_response (request, &message, NULL);
+      ret = ssh_read_response (request, &message, -1);
       switch (ret)
         {
           case FILESIZE:
@@ -422,7 +576,7 @@ ssh_get_file (gftp_request * request, const char *filename, FILE * fd,
 
 
 static int
-ssh_put_file (gftp_request * request, const char *filename, FILE * fd,
+ssh_put_file (gftp_request * request, const char *filename, int fd,
               off_t startsize, off_t totalsize)
 {
   ssh_message message;
@@ -444,7 +598,7 @@ ssh_put_file (gftp_request * request, const char *filename, FILE * fd,
   if (ssh_send_command (request, FILENAME, filename, strlen (filename) + 1) < 0)
     return (-1);
 
-  if (ssh_read_response (request, &message, NULL) < 0)
+  if (ssh_read_response (request, &message, -1) < 0)
     {
       g_free (message.data);
       return (-1);
@@ -470,8 +624,10 @@ ssh_put_file (gftp_request * request, const char *filename, FILE * fd,
   if (startsize > 0)
     {
       startsize = htonl (startsize);
-      /* FIXME - warning on IRIX - pointer from integer of different size */
-      if (ssh_send_command (request, SKIPBYTES, GUINT_TO_POINTER (startsize), 4) < 0)
+      /* This protocol only supports files up to 2 gig in size. I truncate
+         the file size here just to suppress compiler warnings  */
+      if (ssh_send_command (request, SKIPBYTES, 
+                            GINT_TO_POINTER ((gint32) startsize), 4) < 0)
         return (-1);
     }
 
@@ -479,13 +635,13 @@ ssh_put_file (gftp_request * request, const char *filename, FILE * fd,
 }
 
 
-static size_t 
+static ssize_t 
 ssh_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
   ssh_message message;
   size_t len;
 
-  switch (ssh_read_response (request, &message, NULL))
+  switch (ssh_read_response (request, &message, -1))
     {
       case DATA:
         len = size > message.len ? message.len : size;
@@ -494,7 +650,7 @@ ssh_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
         return (len);
       case ENDDATA:
         g_free (message.data);
-        if (ssh_read_response (request, &message, NULL) < 0)
+        if (ssh_read_response (request, &message, -1) < 0)
           {
             g_free (message.data);
             return (-1);
@@ -514,7 +670,7 @@ ssh_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
 }
 
 
-static size_t 
+static ssize_t 
 ssh_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
   if (size == 0)
@@ -536,7 +692,6 @@ ssh_end_transfer (gftp_request * request)
 
   g_return_val_if_fail (request != NULL, -2);
   g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
-  g_return_val_if_fail (request->datafd != NULL, -2);
 
   params = request->protocol_data;
   if (params->buffer)
@@ -548,6 +703,21 @@ ssh_end_transfer (gftp_request * request)
 }
 
 
+static size_t 
+ssh_remove_spaces ( char *string )
+{
+  size_t len;
+  char *pos;
+
+  for (pos = string, len = 0; *pos != '\0'; len++, pos++)
+    {
+      if (*pos == ' ')
+        *pos = '\0';
+    }
+  return (len);
+}
+
+
 static int
 ssh_list_files (gftp_request * request)
 {
@@ -565,7 +735,7 @@ ssh_list_files (gftp_request * request)
   request->logging_function (gftp_logging_misc, request->user_data,
 			     _("Retrieving directory listing...\n"));
 
-  tempstr = g_strconcat ("/bin/ls -al", NULL);
+  tempstr = g_strdup ("/bin/ls -al");
   len = ssh_remove_spaces (tempstr);
 
   if (ssh_send_command (request, EXEC, tempstr, len) < 0) 
@@ -575,7 +745,7 @@ ssh_list_files (gftp_request * request)
     }
   g_free (tempstr);
 
-  if (ssh_read_response (request, &message, NULL) != STREAM)
+  if (ssh_read_response (request, &message, -1) != STREAM)
     {
       g_free (message.data);
       request->logging_function (gftp_logging_misc, request->user_data,
@@ -590,7 +760,7 @@ ssh_list_files (gftp_request * request)
 
 
 static int
-ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
+ssh_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 {
   char *tempstr, *pos;
   ssh_message message;
@@ -715,55 +885,6 @@ ssh_get_next_file (gftp_request * request, gftp_file * fle, FILE * fd)
 }
 
 
-static int
-ssh_chdir (gftp_request * request, const char *directory)
-{
-  ssh_message message;
-  int ret;
-
-  g_return_val_if_fail (request != NULL, -2);
-  g_return_val_if_fail (request->protonum == GFTP_SSH_NUM, -2);
-
-  if (directory != NULL && *directory != '\0')
-    {  
-      if (ssh_send_command (request, CHDIR, directory, 
-                            strlen (directory) + 1) < 0) 
-        return (-1);
-
-      if ((ret = ssh_read_response (request, &message, NULL)) != SUCCESS)
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-			    _("Could not change remote directory to %s: %s\n"),
-	  	            directory, (char *) message.data);
-          g_free (message.data);
-          return (-1);
-        }
-      g_free (message.data);
-    }
-
-  if (directory != request->directory)
-    {
-      if (ssh_send_command (request, GETDIR, NULL, 0) < 0)
-        return (-1);
-
-      if (ssh_read_response (request, &message, NULL) != TELLDIR)
-        {
-          request->logging_function (gftp_logging_error, request->user_data,
-                            _("Could not get current working directory: %s\n"),
-		  	    (char *) message.data);
-          g_free (message.data);
-          return (-1);
-        }
-
-      if (request->directory)
-        g_free (request->directory);
-      request->directory = message.data;
-    }
-
-  return (0);
-}
-
-
 static char *
 ssh_exec (gftp_request * request, const char *command, size_t len)
 {
@@ -781,7 +902,7 @@ ssh_exec (gftp_request * request, const char *command, size_t len)
 
   while (1)
     {
-      ret = ssh_read_response (request, &message, NULL);
+      ret = ssh_read_response (request, &message, -1);
       switch (ret)
         {
           case -1: 
@@ -989,250 +1110,67 @@ ssh_chmod (gftp_request * request, const char *file, int mode)
 }
 
 
-static int
-ssh_send_command (gftp_request * request, int cmdnum, const char *command, 
-                  size_t len)
-{
-  ssize_t wrote;
-  ssh_parms * params;
-  char *buf;
-  int clen;
-
-  params = request->protocol_data;
-  clen = htonl (len);
-  buf = g_malloc (len + 6);
-  buf[0] = params->channel;
-  buf[1] = cmdnum;
-  memcpy (&buf[2], &clen, 4);
-  if (command)
-    memcpy (&buf[6], command, len);
-  ssh_log_command (request, params->channel, cmdnum, command, len, 1);
-
-  wrote = fwrite (buf, 1, len + 6, request->sockfd_write);
-  if (ferror (request->sockfd_write))
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                               _("Error: Could not write to socket: %s\n"),
-                               g_strerror (errno));
-      gftp_disconnect (request);
-      g_free (buf);
-      return (-1);
-    }
-
-  g_free (buf);
-
-  fflush (request->sockfd_write);
-  if (ferror (request->sockfd_write))
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Error: Could not write to socket: %s\n"),
-                                 g_strerror (errno));
-      gftp_disconnect (request);
-      return (-1);
-    }
-  return 0;
-
-}
-
-static int
-ssh_read_response (gftp_request * request, ssh_message *message, FILE * fd)
-{
-  char buf[6];
-
-  if (ssh_read_message (request, buf, 6, fd) == NULL)
-    return (-1);
-
-  message->channel = buf[0];
-  message->command = buf[1];
-  memcpy (&message->len, buf + 2, 4);
-  message->len = ntohl (message->len);
-  if (message->len > 8192)
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                             _("Error: Message size %d too big from server\n"),
-                             message->len);
-      memset (message, 0, sizeof (*message));
-      gftp_disconnect (request);
-      return (-1);
-    }
-  
-  message->data = g_malloc (message->len + 1); 
-
-  if (message->len > 0 && ssh_read_message (request, 
-                           (char *) message->data, message->len, fd) == NULL)
-    return (-1);
-
-  ((char *) message->data)[message->len] = '\0'; 
-  ssh_log_command (request, message->channel, message->command, message->data,
-                   message->len, 0);
-  return (message->command);
-}
-
-
-static char *
-ssh_read_message (gftp_request * request, char *buf, size_t len, FILE * fd)
-{
-  if (fd == NULL)
-    fd = request->sockfd;
-
-  fread (buf, len, 1, fd);
-  if (ferror (fd))
-    {
-      request->logging_function (gftp_logging_error, request->user_data,
-                                 _("Error: Could not read from socket: %s\n"),
-                                 g_strerror (errno));
-      gftp_disconnect (request);
-      return (NULL);
-    }
-
-  return (buf);
-}
-
-
-static char *
-ssh_read_line (gftp_request * request)
-{
-  char *retstr, *pos, tempchar;
-  ssh_parms *buffer;
-
-  pos = NULL;
-  buffer = request->protocol_data;
-  if (!buffer->enddata && (pos = strchr (buffer->pos, '\n')) == NULL)
-    return (NULL);
-
-  if (pos == NULL)
-    {
-      pos = buffer->pos + strlen (buffer->pos) - 1;
-      tempchar = '\0';
-    }
-  else
-    tempchar = pos + 1 == '\0' ? '\0' : '1';
-
-  if (*(pos-1) == '\r') 
-    *(pos-1) = '\0';
-  else 
-    *pos = '\0';
-
-  retstr = g_malloc (strlen (buffer->pos) + 1);
-  strcpy (retstr, buffer->pos);
-
-  if (tempchar != '\0' && *buffer->pos != '\0')
-    {
-      buffer->pos = pos + 1;
-      while (*buffer->pos == '\r' || *buffer->pos == '\n')
-        buffer->pos++;
-    }
-  else
-    {
-      g_free (buffer->buffer);
-      buffer->buffer = buffer->pos = NULL;
-    }
-  return (retstr);
-}
-
-
 static void
-ssh_log_command (gftp_request * request, int channel, int cmdnum, 
-                 const char *command, size_t len, int direction)
+ssh_set_config_options (gftp_request * request)
 {
-  const char *pos;
-  char *tempstr;
-  int ok;
-
-  switch (cmdnum)
+  if (request->sftpserv_path != NULL)
     {
-      case CHDIR:
-        tempstr = "CHDIR ";
-        break;
-      case GETDIR:
-        tempstr = "GETDIR ";
-        break;
-      case TELLDIR:
-        tempstr = "TELLDIR ";
-        break;
-      case SENDFILE:
-        tempstr = "SENDFILE ";
-        break;
-      case FILESIZE:
-        tempstr = "FILESIZE ";
-        break;
-      case FILEMODE:
-        tempstr = "FILEMODE ";
-        break;
-      case ENDDATA:
-        tempstr = "ENDDATA ";
-        break;
-      case FILEOK:
-        tempstr = "FILEOK ";
-        break;
-      case STREAM:
-        tempstr = "STREAM ";
-        break;
-      case REQUEST:
-        tempstr = "REQUEST ";
-        break;
-      case FILENAME:
-        tempstr = "FILENAME ";
-        break;
-      case EXEC:
-        tempstr = "EXEC ";
-        break;
-      case SKIPBYTES:
-        tempstr = "SKIPBYTES ";
-        break;
-      case ERROR:
-        tempstr = "ERROR: ";
-        break;
-      case SUCCESS:
-        tempstr = "SUCCESS ";
-        break;
-      case CLOSE:
-        tempstr = "CLOSE ";
-        break;
-      case SSH_VERSION:
-        tempstr = "VERSION ";
-        break;
-      case CANCEL:
-        tempstr = "CANCEL ";
-        break;
-      case FILETIME:
-        tempstr = "FILETIME ";
-        break;
-      default:
+      if (ssh1_sftp_path != NULL &&
+          strcmp (ssh1_sftp_path, request->sftpserv_path) == 0)
         return;
+
+      g_free (request->sftpserv_path);
+      request->sftpserv_path = NULL;
     }
 
-  ok = 0;
-  if (command)
-    {
-      for (pos = command; pos < command + len; pos++)
-        {
-          if (*pos == '\0')
-            {
-              ok = 1;
-              break;
-            }
-        }
-    }
+  if (ssh1_sftp_path != NULL)
+    request->sftpserv_path = g_strdup (ssh1_sftp_path);
 
-  request->logging_function (direction == GFTP_DIRECTION_DOWNLOAD ? 
-                                 gftp_logging_send : gftp_logging_recv,
-                             request->user_data, "%d: %s %s\n", channel, 
-                             tempstr, ok ? command : "");
+  request->need_userpass = ssh_need_userpass;
 }
 
 
-static size_t 
-ssh_remove_spaces ( char *string )
+void
+ssh_init (gftp_request * request)
 {
-  size_t len;
-  char *pos;
+  g_return_if_fail (request != NULL);
 
-  for (pos = string, len = 0; *pos != '\0'; len++, pos++)
-    {
-      if (*pos == ' ')
-        *pos = '\0';
-    }
-  return (len);
+  request->protonum = GFTP_SSH_NUM;
+  request->init = ssh_init;
+  request->destroy = ssh_destroy;
+  request->connect = ssh_connect;
+  request->disconnect = ssh_disconnect;
+  request->get_file = ssh_get_file;
+  request->put_file = ssh_put_file;
+  request->transfer_file = NULL;
+  request->get_next_file_chunk = ssh_get_next_file_chunk;
+  request->put_next_file_chunk = ssh_put_next_file_chunk;
+  request->end_transfer = ssh_end_transfer;
+  request->abort_transfer = NULL; /* FIXME */
+  request->list_files = ssh_list_files;
+  request->get_next_file = ssh_get_next_file;
+  request->set_data_type = NULL;
+  request->get_file_size = NULL;
+  request->chdir = ssh_chdir;
+  request->rmdir = ssh_rmdir;
+  request->rmfile = ssh_rmfile;
+  request->mkdir = ssh_mkdir;
+  request->rename = ssh_rename;
+  request->chmod = ssh_chmod;
+  request->set_file_time = NULL;
+  request->site = NULL;
+  request->parse_url = NULL;
+  request->set_config_options = ssh_set_config_options;
+  request->swap_socks = NULL;
+  request->url_prefix = "ssh";
+  request->protocol_name = "SSH";
+  request->need_hostport = 1;
+  request->need_userpass = ssh_need_userpass;
+  request->use_cache = 1;
+  request->use_threads = 1;
+  request->always_connected = 0;
+  request->protocol_data = g_malloc0 (sizeof (ssh_parms));
+  gftp_set_config_options (request);
 }
 
+
