@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: bookmarks.c,v 1.8 2002/11/06 02:20:25 masneyb Exp $";
+static const char cvsid[] = "$Id: bookmarks.c,v 1.9 2002/12/03 02:36:37 masneyb Exp $";
 
 static GtkWidget * bm_hostedit, * bm_portedit, * bm_localdiredit,
   * bm_remotediredit, * bm_useredit, * bm_passedit, * bm_acctedit, * anon_chk,
@@ -91,11 +91,6 @@ run_bookmark (gpointer data)
   if (!gftp_protocols[i].name)
     gftp_protocols[0].init (current_wdata->request);
 
-  /* If we're using one of the SSH protocols, override the value of 
-     ssh_need_userpass in the config file */
-  if (strncmp (tempentry->protocol, "SSH", 3) == 0)
-    current_wdata->request->need_userpass = 1;
-
   ftp_connect (current_wdata, current_wdata->request, 1);
 }
 
