@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: dnd.c,v 1.9 2002/11/27 14:29:55 masneyb Exp $";
+static const char cvsid[] = "$Id: dnd.c,v 1.10 2002/12/04 02:35:44 masneyb Exp $";
 
 
 static int
@@ -84,9 +84,6 @@ dnd_remote_file (char *url, gftp_window_data * wdata)
   
   newfle->destfile = g_strconcat (GFTP_GET_DIRECTORY (wdata->request),
                                      "/", pos, NULL);
-  newfle->ascii = gftp_get_file_transfer_mode (newfle->file, 
-                                wdata->request->data_type) == GFTP_TYPE_ASCII;
-
   templist = g_malloc0 (sizeof (*templist));
   templist->data = newfle;
   templist->next = NULL;
