@@ -18,21 +18,7 @@
 /*****************************************************************************/
 
 #include <gftp-gtk.h>
-
-static void do_openurl 				( GtkWidget * widget, 
-						  gftp_dialog_data * data );
-static void dochange_filespec 			( GtkWidget * widget, 
-						  gftp_dialog_data * data );
-static void dosite 				( GtkWidget * widget, 
-						  gftp_dialog_data * data );
-static int do_change_dir			( gftp_window_data * wdata,
-						  char * directory );
-static void *do_change_dir_thread 		( void * data );
-static void dosavelog 				( GtkWidget * widget, 
-						  GtkFileSelection * fs );
-
-static RETSIGTYPE sig_chdirquit                 ( int signo );
-static sigjmp_buf chdirenvir;
+static const char cvsid[] = "$Id: menu-items.c,v 1.12 2002/11/11 23:16:12 masneyb Exp $";
 
 void
 change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
@@ -93,22 +79,11 @@ tb_openurl_dialog (gpointer data)
 }
 
 
-void
-openurl_dialog (gpointer data)
-{
-  MakeEditDialog (_("Connect via URL"), _("Enter ftp url to connect to"),
-                  NULL, 1, 1, NULL, _("Connect"), do_openurl, data,
-                  _("  Cancel  "), NULL, NULL);
-}
-
-
 static void
-do_openurl (GtkWidget * widget, gftp_dialog_data * data)
+do_openurl (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
-  gftp_window_data * wdata;
   const char *tempstr;
 
-  wdata = data->data;
   if (current_wdata->request->stopable)
     {
       ftp_log (gftp_logging_misc, NULL,
@@ -117,7 +92,7 @@ do_openurl (GtkWidget * widget, gftp_dialog_data * data)
       return;
     }
 
-  tempstr = gtk_entry_get_text (GTK_ENTRY (data->edit));
+  tempstr = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*tempstr == '\0')
     {
       ftp_log (gftp_logging_misc, NULL,
@@ -130,15 +105,24 @@ do_openurl (GtkWidget * widget, gftp_dialog_data * data)
 
   if (gftp_parse_url (wdata->request, tempstr) == 0)
     {
-      gtk_widget_destroy (data->dialog);
+      gtk_widget_destroy (ddata->dialog);
       ftp_connect (wdata, wdata->request, 1);
     }
   else
     {
-      gtk_widget_destroy (data->dialog);
+      gtk_widget_destroy (ddata->dialog);
       ftp_log (gftp_logging_misc, NULL, _("Could not parse URL %s\n"), tempstr);    
     }
-  data->dialog = NULL; 
+  ddata->dialog = NULL; 
+}
+
+
+void
+openurl_dialog (gpointer data)
+{
+  MakeEditDialog (_("Connect via URL"), _("Enter ftp url to connect to"),
+                  NULL, 1, NULL, gftp_dialog_button_connect, do_openurl, data,
+                  NULL, NULL);
 }
 
 
@@ -148,40 +132,24 @@ disconnect (gpointer data)
   gftp_window_data * wdata;
 
   wdata = data;
+  gftp_delete_cache_entry (wdata->request, 1);
   gftp_disconnect (wdata->request);
   remove_files_window (wdata);
   update_window (wdata);
 }
 
 
-void 
-change_filespec (gpointer data)
-{
-  gftp_window_data * wdata;
-
-  wdata = data;
-  if (!check_status (_("Change Filespec"), wdata, 0, 0, 0, 1))
-    return;
-
-  MakeEditDialog (_("Change Filespec"), _("Enter the new file specification"),
-                  wdata->filespec, 1, 1, NULL, _("Change"), dochange_filespec,
-                  wdata, _("  Cancel  "), NULL, NULL);
-}
-
-
 static void
-dochange_filespec (GtkWidget * widget, gftp_dialog_data * data)
+dochange_filespec (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
   GList * templist, * filelist;
   gftp_file * tempfle;
-  gftp_window_data * wdata;
   const char *edttext;
   int num;
 
-  wdata = data->data;
   wdata->show_selected = 0;
 
-  edttext = gtk_entry_get_text (GTK_ENTRY (data->edit));
+  edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*edttext == '\0')
     {
       ftp_log (gftp_logging_misc, NULL,
@@ -217,6 +185,116 @@ dochange_filespec (GtkWidget * widget, gftp_dialog_data * data)
 }
 
 
+void 
+change_filespec (gpointer data)
+{
+  gftp_window_data * wdata;
+
+  wdata = data;
+  if (!check_status (_("Change Filespec"), wdata, 0, 0, 0, 1))
+    return;
+
+  MakeEditDialog (_("Change Filespec"), _("Enter the new file specification"),
+                  wdata->filespec, 1, NULL, gftp_dialog_button_change, 
+                  dochange_filespec, wdata, NULL, NULL);
+}
+
+
+static void
+destroy_save_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
+{
+  gtk_widget_destroy (str->filew);
+  g_free (str);
+}
+
+
+static void
+dosave_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
+{
+  const char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
+    "Aug", "Sep", "Oct", "Nov", "Dec" };
+  const char *filename;
+  struct tm *lt;
+  gftp_file * tempfle;
+  GList * templist;
+  FILE * fd;
+  time_t t;
+
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (str->filew));
+  if ((fd = fopen (filename, "w")) == NULL)
+    {
+      ftp_log (gftp_logging_misc, NULL, 
+               _("Error: Cannot open %s for writing: %s\n"), filename, 
+               g_strerror (errno));
+      return;
+    }
+
+  time (&t);
+
+  for (templist = str->wdata->files; 
+       templist->next != NULL;
+       templist = templist->next)
+    {
+      tempfle = templist->data;
+      fprintf (fd, "%10s %8s %8s ", tempfle->attribs, tempfle->user, 
+               tempfle->group);
+      if (tempfle->attribs && (*tempfle->attribs == 'b' || 
+                               *tempfle->attribs == 'c'))
+        fprintf (fd, "%5d, %4d", (((unsigned int) tempfle->size) >> 16) & 0xFF,
+                                 ((unsigned int) tempfle->size) & 0xFF);
+      else
+        {
+#if defined (_LARGEFILE_SOURCE)
+          fprintf (fd, "%11lld", tempfle->size);
+#else
+          fprintf (fd, "%11ld", tempfle->size);
+#endif
+        }
+
+      lt = localtime (&tempfle->datetime);
+
+      if (tempfle->datetime > t ||
+          t - 3600*24*90 > tempfle->datetime)
+        fprintf (fd, " %s %2d  %4d", months[lt->tm_mon], lt->tm_mday, 
+                 lt->tm_year + 1900);
+      else
+        fprintf (fd, " %s %2d %02d:%02d", months[lt->tm_mon], lt->tm_mday, 
+                 lt->tm_hour, lt->tm_min);
+
+      fprintf (fd, " %s\n", tempfle->file);
+    }
+
+  fclose (fd);
+}
+
+
+void 
+save_directory_listing (gpointer data)
+{
+  gftp_save_dir_struct * str;
+  GtkWidget *filew;
+
+  filew = gtk_file_selection_new (_("Save Directory Listing"));
+
+  str = g_malloc (sizeof (*str));
+  str->filew = filew;
+  str->wdata = data;
+
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
+                      "clicked", GTK_SIGNAL_FUNC (dosave_directory_listing), 
+                      str);
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
+                      "clicked", 
+                      GTK_SIGNAL_FUNC (destroy_save_directory_listing), str);
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), 
+                      "clicked", 
+                      GTK_SIGNAL_FUNC (destroy_save_directory_listing), str);
+
+  gtk_window_set_wmclass (GTK_WINDOW(filew), "Save Directory Listing", "gFTP");
+  gtk_widget_show (filew);
+}
+
+
 void
 show_selected (gpointer data)
 {
@@ -275,7 +353,7 @@ selectallfiles (gpointer data)
   wdata->show_selected = 0;
   gtk_clist_freeze (GTK_CLIST (wdata->listbox));
   i = 0;
-  templist = g_list_first (wdata->files);
+  templist = wdata->files;
   while (templist != NULL)
     {
       tempfle = (gftp_file *) templist->data;
@@ -304,28 +382,12 @@ deselectall (gpointer data)
 }
 
 
-void
-site_dialog (gpointer data)
-{
-  gftp_window_data * wdata;
-
-  wdata = data;
-  if (!check_status (_("Site"), wdata, 0, 0, 0, wdata->request->site != NULL))
-    return;
-
-  MakeEditDialog (_("Site"), _("Enter site-specific command"), NULL, 1, 1,
-                  NULL, _("OK"), dosite, wdata, _("  Cancel  "), NULL, NULL);
-}
-
-
 static void
-dosite (GtkWidget * widget, gftp_dialog_data * data)
+dosite (gftp_window_data * wdata, gftp_dialog_data * ddata)
 {
-  gftp_window_data * wdata;
   const char *edttext;
 
-  wdata = data->data;
-  edttext = gtk_entry_get_text (GTK_ENTRY (data->edit));
+  edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*edttext == '\0')
     {
       ftp_log (gftp_logging_misc, NULL,
@@ -342,6 +404,97 @@ dosite (GtkWidget * widget, gftp_dialog_data * data)
 }
 
 
+void
+site_dialog (gpointer data)
+{
+  gftp_window_data * wdata;
+
+  wdata = data;
+  if (!check_status (_("Site"), wdata, 0, 0, 0, wdata->request->site != NULL))
+    return;
+
+  MakeEditDialog (_("Site"), _("Enter site-specific command"), NULL, 1,
+                  NULL, gftp_dialog_button_ok, dosite, wdata, NULL, NULL);
+}
+
+
+static void *
+do_change_dir_thread (void * data)
+{
+  gftp_window_data * wdata;
+  int success, sj;
+
+  wdata = data;
+
+  if (wdata->request->use_threads)
+    {
+      sj = sigsetjmp (jmp_environment, 1);
+      use_jmp_environment = 1;
+    }
+  else
+    sj = 0;
+
+  success = 0;
+  if (sj == 0) 
+    {
+      if (wdata->request->network_timeout > 0)
+        alarm (wdata->request->network_timeout);
+      success = gftp_set_directory (wdata->request, wdata->request->directory);
+      alarm (0);
+    }
+  else
+    {
+      gftp_disconnect (wdata->request);
+      wdata->request->logging_function (gftp_logging_error,
+                                        wdata->request->user_data,
+                                        _("Operation canceled\n"));
+    }
+
+  if (wdata->request->use_threads)
+    use_jmp_environment = 0;
+
+  wdata->request->stopable = 0;
+  return ((void *) success);
+}
+
+
+static int
+do_change_dir (gftp_window_data * wdata, char *directory)
+{
+  char *olddir;
+  int ret;
+
+  if (directory != wdata->request->directory)
+    {
+      olddir = wdata->request->directory;
+      wdata->request->directory = g_malloc (strlen (directory) + 1);
+      strcpy (wdata->request->directory, directory);
+    }
+  else
+    olddir = NULL;
+
+  ret = (int) generic_thread (do_change_dir_thread, wdata);
+
+  if (!GFTP_IS_CONNECTED (wdata->request))
+    {
+      disconnect (wdata);
+      if (olddir != NULL)
+        g_free (olddir);
+      return (-2);
+    }
+
+  if (ret != 0)
+    {
+      g_free (wdata->request->directory);
+      wdata->request->directory = olddir;
+    }
+  else
+    g_free (olddir);
+
+  return (ret);
+}
+
+
 int
 chdir_edit (GtkWidget * widget, gpointer data)
 {
@@ -395,7 +548,7 @@ chdir_dialog (gpointer data)
                      wdata->request->chdir != NULL))
     return (0);
 
-  filelist = g_list_first (wdata->files);
+  filelist = wdata->files;
   templist = GTK_CLIST (wdata->listbox)->selection;
   num = 0;
   templist = get_next_selection (templist, &filelist, &num);
@@ -425,102 +578,11 @@ chdir_dialog (gpointer data)
 }
 
 
-static int
-do_change_dir (gftp_window_data * wdata, char *directory)
-{
-  char *olddir;
-  int ret;
-
-  if (directory != wdata->request->directory)
-    {
-      olddir = wdata->request->directory;
-      wdata->request->directory = g_malloc (strlen (directory) + 1);
-      strcpy (wdata->request->directory, directory);
-    }
-  else
-    olddir = NULL;
-
-  ret = (int) generic_thread (do_change_dir_thread, wdata);
-
-  if (!GFTP_IS_CONNECTED (wdata->request))
-    {
-      disconnect (wdata);
-      if (olddir != NULL)
-        g_free (olddir);
-      return (-2);
-    }
-
-  if (ret != 0)
-    {
-      g_free (wdata->request->directory);
-      wdata->request->directory = olddir;
-    }
-  else
-    g_free (olddir);
-
-  return (ret);
-}
-
-
-static void *
-do_change_dir_thread (void * data)
-{
-  gftp_window_data * wdata;
-  int success, sj;
-
-  wdata = data;
-  wdata->request->user_data = (void *) 0x01;
-
-  if (wdata->request->use_threads)
-    {
-      sj = sigsetjmp (chdirenvir, 1);
-      signal (SIGINT, sig_chdirquit);
-      signal (SIGALRM, sig_chdirquit);
-    }
-  else
-    sj = 0;
-
-  success = 0;
-  if (sj == 0) 
-    {
-      if (wdata->request->network_timeout > 0)
-        alarm (wdata->request->network_timeout);
-      success = gftp_set_directory (wdata->request, wdata->request->directory);
-      alarm (0);
-    }
-  else
-    {
-      gftp_disconnect (wdata->request);
-      wdata->request->logging_function (gftp_logging_error,
-                                        wdata->request->user_data,
-                                        _("Operation canceled\n"));
-    }
-
-  if (wdata->request->use_threads)
-    {
-      signal (SIGINT, SIG_DFL);
-      signal (SIGALRM, SIG_IGN);
-    }
-
-  wdata->request->user_data = NULL;
-  wdata->request->stopable = 0;
-  return ((void *) success);
-}
-
-
-static RETSIGTYPE
-sig_chdirquit (int signo)
-{
-  signal (signo, sig_chdirquit);
-  siglongjmp (chdirenvir, signo == SIGINT ? 1 : 2);
-}
-
-
 void 
 clearlog (gpointer data)
 {
   guint len;
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   len = gtk_text_get_length (GTK_TEXT (logwdw));
   gtk_text_set_point (GTK_TEXT (logwdw), len);
   gtk_text_backward_delete (GTK_TEXT (logwdw), len);
@@ -544,23 +606,24 @@ viewlog (gpointer data)
   guint textlen;
   ssize_t len;
   int fd;
-#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+#if GTK_MAJOR_VERSION > 1
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
 #endif
 
-  tempstr = g_strconcat (tmp_directory, "/gftp-view.XXXXXXXXXX", NULL);
+  tempstr = g_strconcat (g_get_tmp_dir (), "/gftp-view.XXXXXXXXXX", NULL);
   if ((fd = mkstemp (tempstr)) < 0)
     {
-      ftp_log (gftp_logging_misc, NULL, _("Error: Cannot open %s: %s\n"),
-	       tempstr, g_strerror (errno));
+      ftp_log (gftp_logging_misc, NULL, 
+               _("Error: Cannot open %s for writing: %s\n"), tempstr, 
+               g_strerror (errno));
       g_free (tempstr); 
       return;
     }
   chmod (tempstr, S_IRUSR | S_IWUSR);
   unlink (tempstr);
   
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   textlen = gtk_text_get_length (GTK_TEXT (logwdw));
   txt = gtk_editable_get_chars (GTK_EDITABLE (logwdw), 0, -1);
 #else
@@ -590,26 +653,6 @@ viewlog (gpointer data)
 }
 
 
-void 
-savelog (gpointer data)
-{
-  GtkWidget *filew;
-
-  filew = gtk_file_selection_new (_("Save Log"));
-
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
-                      "clicked", GTK_SIGNAL_FUNC (dosavelog), filew);
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
-                             "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-                             GTK_OBJECT (filew));
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (filew));
-
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "gftp.log");
-  gtk_window_set_wmclass (GTK_WINDOW(filew), "Save Log", "gFTP");
-  gtk_widget_show (filew);
-}
-
-
 static void
 dosavelog (GtkWidget * widget, GtkFileSelection * fs)
 {
@@ -619,7 +662,7 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
   ssize_t len;
   FILE *fd;
   int ok;
-#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+#if GTK_MAJOR_VERSION > 1
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
 #endif
@@ -627,12 +670,13 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
   filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
   if ((fd = fopen (filename, "w")) == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL, _("Error: Cannot open %s: %s\n"),
-               filename, g_strerror (errno));
+      ftp_log (gftp_logging_misc, NULL, 
+               _("Error: Cannot open %s for writing: %s\n"), filename, 
+               g_strerror (errno));
       return;
     }
 
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   textlen = gtk_text_get_length (GTK_TEXT (logwdw));
   txt = gtk_editable_get_chars (GTK_EDITABLE (logwdw), 0, -1);
 #else
@@ -667,6 +711,26 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
 }
 
 
+void 
+savelog (gpointer data)
+{
+  GtkWidget *filew;
+
+  filew = gtk_file_selection_new (_("Save Log"));
+
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
+                      "clicked", GTK_SIGNAL_FUNC (dosavelog), filew);
+  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
+                             "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
+                             GTK_OBJECT (filew));
+  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (filew));
+
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "gftp.log");
+  gtk_window_set_wmclass (GTK_WINDOW(filew), "Save Log", "gFTP");
+  gtk_widget_show (filew);
+}
+
+
 void 
 clear_cache (gpointer data)
 {
@@ -682,7 +746,7 @@ about_dialog (gpointer data)
   char *tempstr, *no_license_agreement, *str, buf[255];
   size_t len;
   FILE * fd;
-#if !(GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2)
+#if GTK_MAJOR_VERSION > 1
   GtkTextBuffer * textbuf;
   GtkTextIter iter;
   guint textlen;
@@ -690,20 +754,31 @@ about_dialog (gpointer data)
 
   no_license_agreement = g_strdup_printf (_("Cannot find the license agreement file COPYING. Please make sure it is in either %s or in %s"), BASE_CONF_DIR, SHARE_DIR);
 
+#if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("About gFTP"));
-  gtk_window_set_wmclass (GTK_WINDOW(dialog), "about", "gFTP");
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER
 			      (GTK_DIALOG (dialog)->action_area), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
   gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
+#else
+  dialog = gtk_dialog_new_with_buttons (_("About gFTP"), NULL, 0,
+                                        GTK_STOCK_CLOSE,
+                                        GTK_RESPONSE_CLOSE,
+                                        NULL);
+#endif
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "about", "gFTP");
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-  gtk_signal_connect_object (GTK_OBJECT (dialog), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
   gtk_widget_realize (dialog);
 
+  if (gftp_icon != NULL)
+    {
+      gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
+                           gftp_icon->bitmap);
+      gdk_window_set_icon_name (dialog->window, _("gFTP Icon"));
+    }
+
   notebook = gtk_notebook_new ();
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE,
 		      TRUE, 0);
@@ -743,7 +818,7 @@ version);
   gtk_box_pack_start (GTK_BOX (box), tempwid, TRUE, TRUE, 0);
   gtk_widget_show (tempwid);
 
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
   view = gtk_text_new (NULL, NULL);
   gtk_text_set_editable (GTK_TEXT (view), FALSE);
   gtk_text_set_word_wrap (GTK_TEXT (view), TRUE);
@@ -784,6 +859,7 @@ version);
 
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box, label);
 
+#if GTK_MAJOR_VERSION == 1
   tempwid = gtk_button_new_with_label (_("  Close  "));
   GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
@@ -793,6 +869,11 @@ version);
 			     GTK_OBJECT (dialog));
   gtk_widget_grab_default (tempwid);
   gtk_widget_show (tempwid);
+#else
+  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+                            G_CALLBACK (gtk_widget_destroy),
+                            GTK_OBJECT (dialog));
+#endif
 
   tempstr = g_strconcat ("/usr/share/common-licenses/GPL", NULL);
   if (access (tempstr, F_OK) != 0)
@@ -805,7 +886,7 @@ version);
           tempstr = expand_path (BASE_CONF_DIR "/COPYING");
 	  if (access (tempstr, F_OK) != 0)
 	    {
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
 	      gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL,
 			       no_license_agreement, -1);
 #else
@@ -821,7 +902,7 @@ version);
 
   if ((fd = fopen (tempstr, "r")) == NULL)
     {
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
       gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL,
 		       no_license_agreement, -1);
 #else
@@ -839,7 +920,7 @@ version);
   while ((len = fread (buf, 1, sizeof (buf) - 1, fd)))
     {
       buf[len] = '\0';
-#if GTK_MAJOR_VERSION == 1 && GTK_MINOR_VERSION == 2
+#if GTK_MAJOR_VERSION == 1
       gtk_text_insert (GTK_TEXT (view), NULL, NULL, NULL, buf, -1);
 #else
       textlen = gtk_text_buffer_get_char_count (textbuf);
