@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include <gftp-gtk.h>
-static const char cvsid[] = "$Id: transfer.c,v 1.22 2002/11/27 14:29:55 masneyb Exp $";
+static const char cvsid[] = "$Id: transfer.c,v 1.24 2002/12/05 00:43:07 masneyb Exp $";
 
 static GtkWidget * dialog;
 
@@ -727,13 +727,12 @@ parse_attribs (char *attribs)
 void * 
 gftp_gtk_transfer_files (void *data)
 {
+  int i, mode, dl_type, tofd, fromfd, old_from_datatype;
   gftp_transfer * transfer;
   char *tempstr, buf[8192];
-  int tofd, fromfd;
   off_t fromsize, total;
   gftp_file * curfle; 
   ssize_t num_read;
-  int i, mode;
 
   pthread_detach (pthread_self ());
   transfer = data;
@@ -741,11 +740,15 @@ gftp_gtk_transfer_files (void *data)
   gettimeofday (&transfer->starttime, NULL);
   memcpy (&transfer->lasttime, &transfer->starttime, 
           sizeof (transfer->lasttime));
+
+  dl_type = transfer->fromreq->data_type;
+  old_from_datatype = transfer->fromreq->data_type;
+
   while (transfer->curfle != NULL)
     {
       pthread_mutex_lock (transfer->structmutex);
       curfle = transfer->curfle->data;
-      transfer->current_file_number++;
+      transfer->current_file_number++; 
       pthread_mutex_unlock (transfer->structmutex);
  
       if (curfle->transfer_action == GFTP_TRANS_ACTION_SKIP)
@@ -810,6 +813,27 @@ gftp_gtk_transfer_files (void *data)
               transfer->total_bytes += curfle->size;
             }
 
+          if (GFTP_IS_CONNECTED (transfer->fromreq) &&
+              GFTP_IS_CONNECTED (transfer->toreq))
+            {
+              if (!curfle->ascii && old_from_datatype == GFTP_TYPE_BINARY)
+                dl_type = GFTP_TYPE_BINARY;
+              else
+                dl_type = GFTP_GET_DATA_TYPE (transfer->fromreq) == GFTP_TYPE_ASCII || curfle->ascii ? GFTP_TYPE_ASCII : GFTP_TYPE_BINARY;
+
+              if (dl_type != transfer->fromreq->data_type)
+                {
+                   if (gftp_set_data_type (transfer->fromreq, dl_type) != 0)
+                     gftp_disconnect (transfer->fromreq);
+                 }
+
+              if (dl_type != transfer->toreq->data_type)
+                {
+                   if (gftp_set_data_type (transfer->toreq, dl_type) != 0)
+                     gftp_disconnect (transfer->toreq);
+                 }
+            }
+
           if (GFTP_IS_CONNECTED (transfer->fromreq) &&
               GFTP_IS_CONNECTED (transfer->toreq))
             {
@@ -865,7 +889,7 @@ gftp_gtk_transfer_files (void *data)
               total += num_read;
               gftp_gtk_calc_kbs (transfer, num_read);
 
-              if (GFTP_GET_DATA_TYPE (transfer->fromreq) == GFTP_TYPE_ASCII)
+              if (dl_type == GFTP_TYPE_ASCII)
                 tempstr = gftp_convert_ascii (buf, &num_read, 1);
               else
                 tempstr = buf;
@@ -879,7 +903,7 @@ gftp_gtk_transfer_files (void *data)
 
               /* We don't have to free tempstr for a download because new 
                  memory is not allocated for it in that case */
-              if (GFTP_GET_DATA_TYPE (transfer->fromreq) == GFTP_TYPE_ASCII && 
+              if (dl_type == GFTP_TYPE_ASCII &&
                   transfer->transfer_direction == GFTP_DIRECTION_UPLOAD)
                 g_free (tempstr);
             }
